==> klayout-0.10.tar.gz: klayout-0.10/src/dbLayout.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "dbLayout.h"
#include "dbMemStatistics.h"


namespace db
{

Layout::Layout (db::Manager *manager)
  : db::CellGraph (manager),
    m_dbu (0.001), m_invalid (0)
{
  // .. nothing yet ..
}

Layout::~Layout ()
{
  for (std::vector<const char *>::const_iterator p = m_cell_names.begin (); p != m_cell_names.end (); ++p) {
    delete [] *p;
  }
  m_cell_names.clear ();
  m_cell_map.clear ();
}

Layout &
Layout::operator= (const Layout &d)
{
  if (&d != this) {

    db::CellGraph::translate (d, shape_repository ());

    for (std::vector<const char *>::const_iterator p = m_cell_names.begin (); p != m_cell_names.end (); ++p) {
      delete *p;
    }
    m_cell_names.clear ();
    m_cell_map.clear ();

    m_cell_names.reserve (d.m_cell_names.size ());

    cell_index_type i = 0;
    for (std::vector<const char *>::const_iterator p = d.m_cell_names.begin (); p != d.m_cell_names.end (); ++p) {
      char *pp = new char [strlen (*p) + 1];
      strcpy (pp, *p);
      m_cell_names.push_back (pp);
      m_cell_map.insert (std::make_pair (pp, i));
      ++i;
    }

    m_dbu = d.m_dbu;
    m_invalid = d.m_invalid;

  }
  return *this;
}

void
Layout::collect_mem_stat (db::MemStatistics &m) const
{
  for (std::vector<const char *>::const_iterator p = m_cell_names.begin (); p != m_cell_names.end (); ++p) {
    size_t l = strlen (*p) + 1;
    m.layout_info (l, l);
  }
  m.layout_info (m_cell_names);
  m.layout_info (m_layer_props);
  m.layout_info (m_cell_map);
  m.layout_info (m_dbu);
  m.shapes_cache (m_shape_repository);
  db::CellGraph::collect_mem_stat (m);
}

bool 
Layout::has_cell (const char *name)
{
  return m_cell_map.find (name) != m_cell_map.end ();
}

std::pair<bool, Layout::cell_index_type> 
Layout::cell_by_name (const char *name)
{
  cell_map_type::iterator c = m_cell_map.find (name);
  if (c != m_cell_map.end ()) {
    return std::make_pair (true, c->second);
  } else {
    return std::make_pair (false, 0);
  }
}

const char *
Layout::cell_name (cell_index_type index) const
{
  tl_assert (index < m_cell_names.size ());
  return m_cell_names [index];
}

Layout::cell_index_type 
Layout::add_cell (const char *name)
{
  //  assert that there is no cell with that name before
  tl_assert (m_cell_map.find (name) == m_cell_map.end ());

  //  create a new cell 
  cell_index_type ci = new_cell ().cell_index ();

  //  enter it's index and cell_name
  char *cp;

  cp = new char [strlen (name) + 1];
  strcpy (cp, name);

  while (m_cell_names.size () < ci) {
    cp = new char [1];
    *cp = 0;
    m_cell_names.push_back (cp);
  }

  if (m_cell_names.size () > ci) {
    delete [] m_cell_names [ci];
    m_cell_names [ci] = cp;
  } else {
    m_cell_names.push_back (cp);
  }

  m_cell_map.insert (std::make_pair (cp, ci));

  return ci;
}

void
Layout::redo (db::Op *op) throw ()
{
  //  .. nothing particular in this class ..
  db::CellGraph::redo (op);
}

void 
Layout::undo (db::Op *op) throw ()
{
  //  .. nothing particular in this class ..
  db::CellGraph::undo (op);
}

}

==> klayout-0.14.tar.gz: klayout-0.14/src/dbLayout.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "dbLayout.h"
#include "dbMemStatistics.h"
#include "tlAssert.h"


namespace db
{

Layout::Layout (db::Manager *manager)
  : db::CellGraph (manager),
    m_dbu (0.001)
{
  // .. nothing yet ..
}

Layout::~Layout ()
{
  //  It is very important to clear the array repository after the CellGraph is destroyed:
  //  only this way the array references are destroyed before the master objects in the array
  //  repository. The assignment of the empty cell graph performs this.
  CellGraph empty;
  CellGraph::operator= (empty);

  for (std::vector<const char *>::const_iterator p = m_cell_names.begin (); p != m_cell_names.end (); ++p) {
    delete [] *p;
  }
  m_cell_names.clear ();
  m_cell_map.clear ();
}

Layout &
Layout::operator= (const Layout &d)
{
  if (&d != this) {

    //  TODO: this currently does not translate the shape and instance arrays into the 
    //  new array repository but rather makes them stand-alone.
    //  TODO: clear array repository
    db::CellGraph::translate (d, shape_repository ());

    for (std::vector<const char *>::const_iterator p = m_cell_names.begin (); p != m_cell_names.end (); ++p) {
      delete *p;
    }
    m_cell_names.clear ();
    m_cell_map.clear ();

    m_cell_names.reserve (d.m_cell_names.size ());

    cell_index_type i = 0;
    for (std::vector<const char *>::const_iterator p = d.m_cell_names.begin (); p != d.m_cell_names.end (); ++p) {
      char *pp = new char [strlen (*p) + 1];
      strcpy (pp, *p);
      m_cell_names.push_back (pp);
      m_cell_map.insert (std::make_pair (pp, i));
      ++i;
    }

    m_dbu = d.m_dbu;

  }
  return *this;
}

void
Layout::collect_mem_stat (db::MemStatistics &m) const
{
  for (std::vector<const char *>::const_iterator p = m_cell_names.begin (); p != m_cell_names.end (); ++p) {
    size_t l = strlen (*p) + 1;
    m.layout_info (l, l);
  }
  m.layout_info (m_cell_names);
  m.layout_info (m_layer_props);
  m.layout_info (m_cell_map);
  m.layout_info (m_dbu);
  m.shapes_cache (m_shape_repository);
  m.shapes_cache (m_array_repository);
  db::CellGraph::collect_mem_stat (m);
}

bool 
Layout::has_cell (const char *name)
{
  return m_cell_map.find (name) != m_cell_map.end ();
}

std::pair<bool, Layout::cell_index_type> 
Layout::cell_by_name (const char *name) const
{
  cell_map_type::const_iterator c = m_cell_map.find (name);
  if (c != m_cell_map.end ()) {
    return std::make_pair (true, c->second);
  } else {
    return std::make_pair (false, 0);
  }
}

const char *
Layout::cell_name (cell_index_type index) const
{
  tl_assert (index < m_cell_names.size ());
  return m_cell_names [index];
}

Layout::cell_index_type 
Layout::add_cell (const char *name)
{
  //  create a new cell 
  cell_index_type ci = new_cell ().cell_index ();

  char b [32];
  if (name == 0) {
    //  if the cell does not have a valid name yet, create a unique one.
    for (int j = 1; ; ++j) {
      sprintf (b, "#%d", int (ci) + j);
      if (m_cell_map.find (b) == m_cell_map.end ()) {
        break;
      }
    } 
    name = b;
  } else if (m_cell_map.find (name) != m_cell_map.end ()) {
    //  check that there is no cell with that name before
    throw tl::Exception (tl::translate ("A cell with that name already exists: %s").c_str (), name);
  }

  //  enter it's index and cell_name
  char *cp;

  cp = new char [strlen (name) + 1];
  strcpy (cp, name);

  while (m_cell_names.size () < ci) {
    cp = new char [1];
    *cp = 0;
    m_cell_names.push_back (cp);
  }

  if (m_cell_names.size () > ci) {
    delete [] m_cell_names [ci];
    m_cell_names [ci] = cp;
  } else {
    m_cell_names.push_back (cp);
  }

  m_cell_map.insert (std::make_pair (cp, ci));

  return ci;
}

void
Layout::rename_cell (cell_index_type id, const char *name)
{
  tl_assert (id < m_cell_names.size ());

  m_cell_map.erase (m_cell_names [id]);

  char *cp = new char [strlen (name) + 1];
  strcpy (cp, name);

  delete [] m_cell_names [id];
  m_cell_names [id] = cp;

  m_cell_map.insert (std::make_pair (cp, id));
}

void
Layout::redo (db::Op *op) throw ()
{
  //  .. nothing particular in this class ..
  db::CellGraph::redo (op);
}

void 
Layout::undo (db::Op *op) throw ()
{
  //  .. nothing particular in this class ..
  db::CellGraph::undo (op);
}

}
